#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/rtxSOAP.h"
#include "rtxsrc/rtxSocket.h"
#include "rtxsrc/rtxStreamMemory.h"
#include "rtxsrc/rtxStreamSocket.h"
#include "rtxsrc/rtxStreamFile.h"
#include "CalcWSDL.h"

#define DEFAULT_URL "http://127.0.0.1:9123"

int main (int argc, char** argv)
{
   OSCTXT       ectxt, dctxt;
   int          i, stat;
   OSSOAPCONN   soapConn;
   OSOCTET*     ptmpbuf = 0;   
   OSBOOL       verbose = FALSE, trace = TRUE, add = TRUE;   
   char*        filename = "input.xml";
   const char* url = DEFAULT_URL;
   

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-s")) add = FALSE;
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else if (!strcmp (argv[i], "-url")) url = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;

         else {
            printf ("usage: client [-s] [-v] [-i <file>] [-url <url>] [-notrace]\n");
            printf ("   -s    send subtract request (instead of add)\n");
            printf ("   -v    verbose mode: print trace info\n");
            printf ("   -i    read input from <file> (default: input.xml)\n");
            printf ("   -url  connect to <url>\n");            
            printf ("   -notrace  do not display trace info\n");            
            return 0;
         }
      }
   }

   /* Init encode and decode context variables */

   stat = rtXmlInitContext (&dctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&dctxt);
      return stat;
   }

   stat = rtXmlInitContext (&ectxt);

   if (0 == stat)
      stat = rtXmlSetEncBufPtr (&ectxt, 0, 0);

   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ectxt);
      return stat;
   }

   if (verbose) {
      rtxSetDiag (&dctxt, 1);
      rtxSetDiag (&ectxt, 1);
   }

   /* One-time socket package initialization (required for Windows only) */

   stat = rtxSocketsInit();
   if (0 != stat) {
      printf ("Socket initialization failed.\n");
      rtxErrSetData (&dctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&dctxt);
      return stat;
   }

   /* --- Request message --- */
   stat = rtxStreamFileOpen (&dctxt, filename, OSRTSTRMF_INPUT);
   if (0 != stat) {
      printf ("Unable to open file input stream.\n");
      rtxErrPrint (&dctxt);
      return stat;
   }

   if (add) {
      Add_Input request;
      /* Decode */
      stat = XmlD_Add_Input (&dctxt, &request);

      if (0 == stat) {
         printf ("Decoding of Request message was successful\n");
         Print_Add_Input ("request", &request);
      }
      else {
         printf ("decode failed\n");
         rtxErrPrint (&dctxt);
         return(1);
      }

      /* Encode */
      stat = XmlE_Add_Input (&ectxt, &request);
      if (0 != stat) {
         printf ("Encoding failed\n");
         rtxErrPrint (&ectxt);
         return stat;
      }
   } 
   else { /* Subtract */
      Subtract_Input request;

      /* Decode */
      stat = XmlD_Subtract_Input (&dctxt, &request);

      if (0 == stat) {
         printf ("Decoding of Request message was successful\n");
         Print_Subtract_Input("request", &request);
      }
      else {
         printf ("decode failed\n");
         rtxErrPrint (&dctxt);
         return(1);
      }

      /* Encode */
      stat = XmlE_Subtract_Input (&ectxt, &request);
      if (0 != stat) {
         printf ("Encoding failed\n");
         rtxErrPrint (&ectxt);
         return stat;
      }
   }

   /* Connect to Calc server */
   rtxSoapInitConn (&soapConn, &dctxt, OSSOAPV1, url);

   soapConn.soapAction = (add) ?
      "\"http://Calc.com/Calc/Add\"" : 
      "\"http://Calc.com/Calc/Subtract\"";   

   stat = rtxSoapConnect (&soapConn);
   if (stat != 0) {
      rtxErrPrint (&dctxt);
      return stat;
   }
   if (trace) printf ("Connection to web service opened\n");

   /* Send request */
   RTDIAG1 (&ectxt, "Request message:\n");
   RTDIAG1 (&ectxt, (const char*)rtXmlGetEncBufPtr(&ectxt));

   stat = rtxSoapSendHttp (&soapConn, rtXmlGetEncBufPtr(&ectxt));
   if (stat != 0) {
      rtxErrPrint (&ectxt);
      return stat;
   }
   if (trace) printf ("HTTP request sent\n");

   /* Wait for response */

   stat = rtxSoapRecvHttp (&soapConn);
   if (stat != 0) {
      rtxErrPrint (&dctxt);
      rtxStreamClose (&dctxt);
      return stat;
   }
   if (trace) printf ("HTTP response received\n");

   /* Read HTTP response into memory */

   stat = rtxSoapRecvHttpContent (&soapConn, &ptmpbuf);
   if (stat != 0) {
      rtxErrPrint (&dctxt);
      return stat;
   }
   RTDIAG1 (&dctxt, "Response message:\n");
   RTDIAG1 (&dctxt, (const char*)ptmpbuf);
   RTDIAG1 (&dctxt, "\n\n");

   /* Close connection */

   rtxStreamClose (&dctxt);

   /* Decode SOAP XML message */

   stat = rtxStreamMemoryAttach 
      (&dctxt, ptmpbuf, soapConn.contentLength, OSRTSTRMF_INPUT);

   if (0 != stat) {
      printf ("rtxStreamMemoryAttach failed\n");
      rtxErrPrint (&dctxt);
      return stat;
   }
   if (add) {
      Add_Output response;
      Add_Fault  fault;
      stat = XmlD_Add_Output (&dctxt, &response, &fault);

      if (0 == stat) {
         if (trace) {
            printf ("Decode was successful\n");
            printf ("Decoded record:\n");
            Print_Add_Output ("AddResponse", &response);
         }
      }
      else if (trace) {
         printf ("decode failed\n");
         rtxErrPrint (&dctxt);
      }
   } 
   else { /* Subtract */
      Subtract_Output response;
      Subtract_Fault  fault;
      stat = XmlD_Subtract_Output (&dctxt, &response, &fault);

      if (0 == stat) {
         if (trace) {
            printf ("Decode was successful\n");
            printf ("Decoded record:\n");
            Print_Subtract_Output ("SubtractResponse", &response);
         }
      }
      else if (trace) {
         printf ("decode failed\n");
         rtxErrPrint (&dctxt);
      }
   }

   rtxFreeContext (&dctxt);
   rtxFreeContext (&ectxt);

   return stat;
}
