/* This test driver program reads an encoded record from a file         */
/* and decodes it and displays the results..                            */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/rtxStreamFile.h"
#include "rtxmlsrc/rtSaxCParser.h"
#include "prod.h"

int main (int argc, char** argv)
{
   ProductType  product;
   OSCTXT	ctxt;
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: reader [ -v ] [ -i <filename>] [ -notrace ]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (verbose)
      rtxSetDiag (&ctxt, 1);

   /* Init PDU data variable */
   Init_ProductType (&ctxt, &product);

   /* Create input source object */
   stat = rtxStreamFileOpen (&ctxt, filename, OSRTSTRMF_INPUT);

   /* Call compiler generated decode function */
   if (stat == 0)
      stat = XmlD_product (&ctxt, &product);

   if (stat == 0) {
      if (trace) {
         printf ("Decode was successful\n");
         printf ("Decoded record:\n");
         Print_ProductType ("product", &product);
      }
   }
   else {
      printf ("decode failed\n");
      rtxErrPrint (&ctxt);
      return -1;
   }

   rtxStreamClose (&ctxt);
   rtxFreeContext (&ctxt);

   return 0;
}

