/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "test.h"

#define MAXMSGLEN 1024

int main (int argc, char** argv)
{
   SmallSizeType mySize;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN];
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;
   
   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [ -v ] [ -o <filename>\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (verbose)
      rtxSetDiag (&ctxt, 1);

   /* Populate structure of generated type */

   Init_SmallSizeType (&ctxt, &mySize);

   mySize._base = 5;
   mySize.system_.value = (OSUTF8CHAR*)"metric";

   /* Encode */

   stat = rtXmlSetEncBufPtr (&ctxt, msgbuf, sizeof(msgbuf));

   if (0 == stat)
      stat = XmlE_smallSize (&ctxt, &mySize);

   if (0 == stat) {
      if (trace) {
         printf ("encoded XML message:\n");
         puts ((char*)msgbuf);
         printf ("\n");
      }
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      return stat;
   }
  
  /* Write the encoded message out to the output file */

  stat = rtXmlWriteToFile (&ctxt, filename);
  if (0 != stat) {
     printf ("Write to file %s failed\n", filename);
     rtxErrPrint (&ctxt);
  } 

   rtxFreeContext (&ctxt);
   return 0;
}
