/* This test driver program reads an encoded record from a file         */
/* and decodes it and displays the results..                            */

#include <stdio.h>
#include <stdlib.h>
#include "test.h"
#include "rtxsrc/rtxStreamFile.h"
#include "rtxmlsrc/osrtxml.h"

int main (int argc, char** argv)
{
   OSCTXT	ctxt;
   OSXMLSTRING  strval;
   int		i, stat;
   const char*  filename = "message.json";
   OSBOOL       verbose = FALSE;

   /* Process command line arguments */
   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else {
            printf ("usage: reader [ -v ] [ -i <filename>] [ -notrace ]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            return 0;
         }
      }
   }

   /* Init context structure */
   stat = rtxInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (verbose) rtxSetDiag (&ctxt, TRUE);

   /* Create input source object */
   stat = rtxStreamFileCreateReader (&ctxt, filename);
   if (0 != stat) {
      printf ("Unable to open file input stream.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   /* Call generated JSON decode function */
   stat = JsonDec_hello (&ctxt, &strval);

   /* Decode complete, check status */
   if (0 == stat) {
      printf ("Decode successful\n");
      printf ("decoded value = '%s'\n", strval.value);
   }
   else {
      printf ("Decode failed\n");
      rtxErrPrint (&ctxt);
   }

   rtxFreeContext (&ctxt);

   return stat;
}
