/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/OSRTFileInputStream.h"
#include "rtxmlsrc/rtXmlCppMsgBuf.h"
#include "test.h"

int main (int argc, char** argv)
{
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: reader [ -v ] [ -i <filename>] [ -notrace ]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   // Create buffer and control objects

   SmallSizeType value;
   OSRTFileInputStream in (filename);
   OSXMLDecodeBuffer decodeBuffer (in);

   if (decodeBuffer.getStatus ()) {
      decodeBuffer.printErrorInfo();
      return decodeBuffer.getStatus ();
   }

   smallSize_CC smallSize (decodeBuffer, value);

   if (verbose) 
      rtxSetDiag (smallSize.getCtxtPtr(), 1);

   // Decode

   stat = smallSize.decode();

   if (0 == stat) {
      if (trace) {
         printf ("decoded XML message:\n");
         smallSize.print ("smallSize");
         printf ("\n");
      }
   }
   else {
      printf ("Encoding failed\n");
      decodeBuffer.printErrorInfo();
      return stat;
   }

   return 0;

}
