
import com.test.*;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import java.io.BufferedInputStream;
import java.io.FileInputStream;

/**
 * This test program reads an encoded record from a file, decodes it,
 * and displays the results.
 */
public class Reader {

   private static String inputFile = "message.xml";
   private static boolean trace = true;

   public static void main(String args[]) {
      //process command line to get the name of the input file
      for(int arg=0; arg < args.length; arg++) {
         if ( args[arg].equals("-i")) {
           inputFile = args[++arg];
         }
         else if ( args[arg].equals("-notrace" ) ) trace = false;
         else {
            System.out.println("Usage: java Reader [-i <filename>] [-notrace]");
            System.out.println("  -i <filename> specifies the input file");
            System.out.println("  -notrace turns off echoing decoded data to stdout");
            System.exit(1);
         }
      }

      //create an XMLStreamReader on the file
      XMLStreamReader reader = null;
      try {
         XMLInputFactory inputFactory = XMLInputFactory.newInstance();
         reader = inputFactory.createXMLStreamReader( inputFile,
                  new BufferedInputStream(
                        new FileInputStream(inputFile) ) );


         //create _CC object
         MakeCall_CC root = new MakeCall_CC();
         root.decodeDocument(reader);

         if ( trace ) {
            //print the decoded content to standard output
            java.io.PrintWriter pw = new java.io.PrintWriter(System.out);
            System.out.println("Decoded value:");
            root.print(pw, "", 0);
            pw.flush();
         }
      }
      catch (Exception e ) {
         if ( reader != null )
            System.out.println( "Exception at reader location: " + reader.getLocation());
         e.printStackTrace();
      }
   }

}