import com.test.*;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.util.List;
import com.objsys.xbinder.runtime.*;

/**
 * This test program encodes a data record and writes the encoded result to an output file.
 */
public class Writer {

   private static DatatypeFactory dtFactory;

   static {
      try {
         dtFactory = DatatypeFactory.newInstance();
      }
      catch ( javax.xml.datatype.DatatypeConfigurationException dce ) 
      { throw new RuntimeException(dce); }
   }

   private static String outputFile = "message.xml";
   private static String charset = "UTF-8";     //default is UTF-8
   private static boolean trace = true;

   public static void main(String args[]) throws Exception 
   {
      //process command line to get the name of the output file
      //process command line to get the name of the input file
      for(int arg = 0; arg < args.length; arg++) {
         if ( args[arg].equals("-o")) {
           outputFile = args[++arg];
         }
         else if ( args[arg].equals("-notrace" ) ) trace = false;
         else {
            System.out.println("Usage: java Writer [-o <filename>] [-notrace]");
            System.out.println("  -o <filename> specifies the output file");
            System.out.println("  -notrace turns off echoing encoded data to stdout");
            System.exit(1);
         }
      }

      //create _CC object
      PersonnelRecord_CC root = new PersonnelRecord_CC();

      //Populate structures
      PersonnelRecord personnelRecord = new PersonnelRecord();
      root.setPersonnelRecord(personnelRecord);
      
      Name name = new Name();
      personnelRecord.setName(name);

      name.setGivenName("John");
      name.setInitial("P");
      name.setFamilyName("Smith");

      personnelRecord.setNumber(51);
      personnelRecord.setTitle("Director");
      personnelRecord.setDateOfHire("19710917");
      
      Name nameOfSpouse = new Name();
      personnelRecord.setNameOfSpouse(nameOfSpouse);

      nameOfSpouse.setGivenName("Mary");
      nameOfSpouse.setInitial("T");
      nameOfSpouse.setFamilyName("Smith");

      PersonnelRecord_children children = new PersonnelRecord_children();
      personnelRecord.setChildren(children);

      List<ChildInformation> element = children.getElement();

      ChildInformation elementN;
      Name elementN_name;

      elementN = new ChildInformation();
      element.add(elementN);

      elementN_name = new Name();
      elementN.setName(elementN_name);

      elementN_name.setGivenName("Ralph");
      elementN_name.setInitial("T");
      elementN_name.setFamilyName("Smith");

      elementN.setDateOfBirth("19571111");


      elementN = new ChildInformation();
      element.add(elementN);

      elementN_name = new Name();
      elementN.setName(elementN_name);

      elementN_name.setGivenName("Susan");
      elementN_name.setInitial("B");
      elementN_name.setFamilyName("Jones");

      elementN.setDateOfBirth("19590717");


      //Encode data
      ByteArrayOutputStream bos = new ByteArrayOutputStream(32768);

      XBXmlEncoder encoder;
      encoder = new XBXmlEncoder(bos, charset);
      encoder.setNamespaces(_employee.namespaceContext);
      root.encodeDocument(encoder);
      encoder.close();

      if ( trace ) {
         //write encoded data to System.out
         System.out.println("Encoded data:\n");
         bos.writeTo(System.out);
         System.out.println();
      }

      //write encoded data to output file
      FileOutputStream fout = new FileOutputStream(outputFile);
      bos.writeTo(fout);
      fout.close();
   }
}
